#include <set>

#include "ttl_define.h"
#include "tt_util.h"
#include "tt_exception.h"

#include "tt_command_line_analyzer.h"

// -- TtCommandLineOption ------------------------------------------------
TtCommandLineOption::TtCommandLineOption( char option, bool is_indispensable ) :
option_( option ),
is_indispensable_( is_indispensable )
{
}

TtCommandLineOption::~TtCommandLineOption()
{
}

char
TtCommandLineOption::GetOption( void ) const
{
  return option_;
}

void
TtCommandLineOption::SetIndispensable( bool flag )
{
  is_indispensable_ = flag;
}

bool
TtCommandLineOption::IsIndispensable( void ) const
{
  return is_indispensable_;
}

// -- TtCommandLineOptionBoolean -----------------------------------------
TtCommandLineOptionBoolean::TtCommandLineOptionBoolean( char option, bool* value ) :
TtCommandLineOptionSingleValue<bool>( option, value, false )
{
}

void
TtCommandLineOptionBoolean::Analyze( const std::vector<std::string> operand )
{
  if ( NOT( operand.empty() ) ) {
    throw TtCommandLineOptionOperandCountException( this->GetOption() );
  }
  *value_ = true;
}

// -- TtCommandLineOptionInteger -----------------------------------------
TtCommandLineOptionInteger::TtCommandLineOptionInteger( char option, int* value, bool is_indispensable ) :
TtCommandLineOptionSingleValue<int>( option, value, is_indispensable )
{
}

void
TtCommandLineOptionInteger::Analyze( const std::vector<std::string> operand )
{
  if ( operand.size() != 1 ) {
    throw TtCommandLineOptionOperandCountException( this->GetOption() );
  }
  int tmp;
  if ( NOT( TtUtil::StringToInteger( operand[0], &tmp, 10 ) ) ) {
    throw TtCommandLineOptionFormatException( this->GetOption() );
  }
  *value_ = tmp;
}

// -- TtCommandLineOptionString ------------------------------------------
TtCommandLineOptionString::TtCommandLineOptionString( char option, std::string* value, bool is_indispensable ) :
TtCommandLineOptionSingleValue<std::string>( option, value, is_indispensable )
{
}

void
TtCommandLineOptionString::Analyze( const std::vector<std::string> operand )
{
  if ( operand.size() != 1 ) {
    throw TtCommandLineOptionOperandCountException( this->GetOption() );
  }
  *value_ = operand[0];
}

// -- TtCommandLineOptionAnalyzer ----------------------------------------
TtCommandLineOptionAnalyzer::TtCommandLineOptionAnalyzer( std::vector<std::string>& rest ) :
rest_( rest ),
table_()
{
}

TtCommandLineOptionAnalyzer::~TtCommandLineOptionAnalyzer()
{
}

void
TtCommandLineOptionAnalyzer::Regist( TtCommandLineOption& option )
{
  if ( table_.find( option.GetOption() ) != table_.end() ) {
    throw TtCommandLineOptionDuplicateDefinitionException( option.GetOption() );
  }
  table_[option.GetOption()] = &option;
}

void
TtCommandLineOptionAnalyzer::AnalyzeForConsole( unsigned int argc, const char** argv )
{
  std::vector<std::string> args;
  for ( unsigned int i = 0; i < argc; ++i ) {
    args.push_back( argv[i] );
  }
  this->Analyze( args );
}

void
TtCommandLineOptionAnalyzer::AnalyzeForWindows( const std::string& args_string )
{
  std::vector<std::string> args;
  std::string tmp;
  enum {
    DEFAULT,
    IN_DOUBLE_QUOTE,
    SEPARATOR,
  } status = DEFAULT;

  for ( unsigned int i = 0; i < args_string.size(); ++i ) {
    const char c = args_string[i];
    switch ( status ) {
    case DEFAULT:
      switch ( c ) {
      case ' ':
      case '\t':
        status = SEPARATOR;
        args.push_back( tmp );
        tmp.clear();
        break;

      case '\"':
        status = IN_DOUBLE_QUOTE;
        break;

      case '\\':
        if ( i + 1 < args_string.size() && args_string[i + 1] == '\"' ) {
          tmp.append( 1, '\"' );
          i += 2;
        }
        break;

      default:
        tmp.append( 1, c );
        break;
      }
      break;

    case IN_DOUBLE_QUOTE:
      switch ( c ) {
      case '\"':
        status = DEFAULT;
        break;

      case '\\':
        if ( i + 1 < args_string.size() && args_string[i + 1] == '\"' ) {
          tmp.append( 1, '\"' );
          i += 2;
        }
        break;

      default:
        tmp.append( 1, c );
        break;
      }
      break;

    case SEPARATOR:
      switch ( c ) {
      case ' ':
      case '\t':
        break;

      default:
        --i;
        status = DEFAULT;
        break;
      }
      break;
    }
  }

  if ( NOT( tmp.empty() ) ) {
    args.push_back( tmp );
  }
  this->Analyze( args );
}


void
TtCommandLineOptionAnalyzer::Analyze( const std::vector<std::string> args )
{
  std::set<char> specified_options;

  for ( std::vector<std::string>::const_iterator it = args.begin(); it != args.end(); ++it ) {
    if ( (*it)[0] == '-' ) {
      if ( it->size() != 2 || table_.find( (*it)[1] ) == table_.end() ) {
        throw TtCommandLineOptionUnknownOptionSpecifiedException( (*it) );
      }
      char option = (*it)[1];

      if ( specified_options.find( option ) != specified_options.end() ) {
        throw TtCommandLineOptionDuplicatedSpecifyException( option );
      }
      specified_options.insert( option );

      std::vector<std::string> operand;
      ++it;
      if ( it != args.end() ) {
        operand.push_back( *it );
      }
      else {
        --it;
      }
      table_[option]->Analyze( operand );
    }
    else {
      rest_.push_back( *it );
    }
  }

  for ( std::map<char, TtCommandLineOption*>::iterator it = table_.begin(); it != table_.end(); ++it ) {
    if ( it->second->IsIndispensable() ) {
      if ( specified_options.find( it->first ) == specified_options.end() ) {
        throw TtCommandLineOptionIndispensableIsNoneException( it->first );
      }
    }
  }
}
