#include <sstream>
#include <typeinfo>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"
#include "tt_util.h"

// -- TtException --------------------------------------------------------
TtException::TtException( void ) :
std::domain_error( "TtException" )
{
}

TtException::~TtException()
{
}


std::string
TtException::Dump( void )
{
  return typeid( *this ).name();
}

// -- TtInternalException ------------------------------------------------
TtInternalException::TtInternalException( const char* file, unsigned int line ) :
file_( file ),
line_( line )
{
}

const char*
TtInternalException::GetFile( void ) const
{
  return file_;
}

unsigned int
TtInternalException::GetLine( void ) const
{
  return line_;
}

std::string
TtInternalException::Dump( void )
{
  std::ostringstream os;
  os << typeid( *this ).name() << " : " << file_ << "(" << line_ << ")";
  return os.str();
}

// -- TtSystemCallException ----------------------------------------------
TtSystemCallException::TtSystemCallException( const char* func_name, unsigned int error_number, const char* file, unsigned line ) :
file_( file ),
line_( line ),
func_name_( func_name ),
error_number_( error_number)
{
}

const char*
TtSystemCallException::GetFile( void ) const
{
  return file_;
}

unsigned int
TtSystemCallException::GetLine( void ) const
{
  return line_;
}


const char*
TtSystemCallException::GetFunctionName( void ) const
{
  return func_name_;
}

unsigned int
TtSystemCallException::GetErrorNumber( void ) const
{
  return error_number_;
}

std::string
TtSystemCallException::Dump( void )
{
  std::ostringstream os;
  os << typeid( *this ).name() << " : " << file_ << "(" << line_ << ")," <<
    func_name_ << "(" << error_number_ << ")";
  return os.str();
}

// -- TtWindowsSystemCallException ---------------------------------------
TtWindowsSystemCallException::TtWindowsSystemCallException( const char* func_name, unsigned int error_number, const char* file, unsigned line ) :
TtSystemCallException( func_name, error_number, file, line )
{
}

std::string
TtWindowsSystemCallException::GetSystemErrorMessage( void ) const
{
  return TtUtil::GetWindowsSystemErrorMessage( this->GetErrorNumber() );
}

std::string
TtWindowsSystemCallException::Dump( void )
{
  std::string tmp = this->TtSystemCallException::Dump();
  tmp.append( "," );
  tmp.append( this->GetSystemErrorMessage() );
  return tmp;
}

// -- TtFileOpenException ------------------------------------------------
TtFileOpenException::TtFileOpenException( const std::string& filename, unsigned int error_number ) :
filename_( filename ),
error_number_( error_number )
{
}

const std::string&
TtFileOpenException::GetFileName( void ) const
{
  return filename_;
}


unsigned int
TtFileOpenException::GetErrorNumber( void ) const
{
  return error_number_;
}

std::string
TtFileOpenException::Dump( void )
{
  std::ostringstream os;
  os << typeid( *this ).name() << " : " << filename_ << "(errno = " << error_number_ << ")";
  return os.str();
}

// -- TtInvalidOperationException ----------------------------------------
TtInvalidOperationException::TtInvalidOperationException( const std::type_info& klass ) :
TtWithCauseClassException( klass )
{
}

std::string
TtInvalidOperationException::Dump( void )
{
  std::ostringstream os;
  os << typeid( *this ).name() << " : " << this->GetCauseClass().name();
  return os.str();
}

// -- TtInvalidFileTypeException -----------------------------------------
TtInvalidFileTypeException::TtInvalidFileTypeException( const std::type_info& klass, const std::string& require_type ) :
TtWithCauseClassException( klass ),
require_type_( require_type )
{
}

const std::string&
TtInvalidFileTypeException::GetRequireType( void ) const
{
  return require_type_;
}

std::string
TtInvalidFileTypeException::Dump( void )
{
  std::ostringstream os;
  os << typeid( *this ).name() << " : " << require_type_;
  return os.str();
}


// -- TtCommandLineOptionOperandCountException ----------------------------------
TtCommandLineOptionOperandCountException::TtCommandLineOptionOperandCountException( char option ) :
TtWithCommandLineOptionException( option )
{
}

// -- TtCommandLineOptionFormatException ----------------------------------
TtCommandLineOptionFormatException::TtCommandLineOptionFormatException( char option ) :
TtWithCommandLineOptionException( option )
{
}

// -- TtCommandLineOptionDuplicateDefinitionException ---------------------
TtCommandLineOptionDuplicateDefinitionException::TtCommandLineOptionDuplicateDefinitionException( char option ) :
TtWithCommandLineOptionException( option )
{
}

// -- TtCommandLineOptionIndispensableIsNoneException ---------------------
TtCommandLineOptionIndispensableIsNoneException::TtCommandLineOptionIndispensableIsNoneException( char option ) :
TtWithCommandLineOptionException( option )
{
}

// -- TtCommandLineOptionUnknownOptionSpecifiedException ------------------
TtCommandLineOptionUnknownOptionSpecifiedException::TtCommandLineOptionUnknownOptionSpecifiedException( const std::string& specified_string ) :
specified_string_( specified_string )
{
}
  
const std::string&
TtCommandLineOptionUnknownOptionSpecifiedException::GetSpecifiedString( void ) const
{
  return specified_string_;
}

// -- TtCommandLineOptionDuplicatedSpecifyException -----------------------
TtCommandLineOptionDuplicatedSpecifyException::TtCommandLineOptionDuplicatedSpecifyException( char option ) :
TtWithCommandLineOptionException( option )
{
}
